package com.miaxis.app.controller.film;

import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.fasterxml.jackson.databind.ObjectMapper;
import com.fasterxml.jackson.databind.node.ObjectNode;
import com.miaxis.common.config.WxpayConfig;
import com.miaxis.common.constant.Constants;
import com.miaxis.common.core.controller.BaseController;
import com.miaxis.common.core.domain.Response;
import com.miaxis.common.core.domain.entity.UserInfo;
import com.miaxis.common.core.page.ResponsePageInfo;
import com.miaxis.common.exception.CustomException;
import com.miaxis.common.utils.SecurityUtils;
import com.miaxis.common.utils.uuid.CommonUtils;
import com.miaxis.feign.dto.FilmDTO;
import com.miaxis.film.domain.FilmOrder;
import com.miaxis.film.dto.*;
import com.miaxis.film.service.IFilmOrderService;
import com.miaxis.question.domain.QuestionInfo;
import com.miaxis.system.service.ISysConfigService;
import com.miaxis.system.service.ISysDictDataService;
import io.swagger.annotations.*;
import lombok.Data;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang.RandomStringUtils;
import org.apache.http.HttpResponse;
import org.apache.http.client.HttpClient;
import org.apache.http.client.methods.HttpGet;
import org.apache.http.client.methods.HttpPost;
import org.apache.http.entity.StringEntity;
import org.apache.http.util.EntityUtils;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.core.io.ClassPathResource;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.web.bind.annotation.*;

import java.io.ByteArrayOutputStream;
import java.io.File;
import java.io.IOException;
import java.security.PrivateKey;
import java.security.Signature;
import java.util.Arrays;
import java.util.Base64;
import java.util.List;
import java.util.stream.Collectors;

import static com.miaxis.common.constant.Constants.FILM_BLACK_LIST;
import static com.miaxis.common.utils.OrderCodeFactory.getOrderCode;

/**
 * 【小程序-客户信息】Controller
 *
 * @author miaxis
 * @date 2021-03-10
 */
@RestController
@RequiredArgsConstructor
@RequestMapping(Constants.STUDENT_PREFIX+"/film/wxpay")
@Api(tags = {"【小程序-电影】"})
@Slf4j
public class FilmController extends BaseController {

    @Autowired
    private HttpClient httpClient;

    @Autowired
    private WxpayConfig wxpayConfig;

    @Autowired
    private IFilmOrderService filmOrderService;

    @Autowired
    private ISysConfigService configService;

    @Autowired
    private ISysDictDataService dictDataService;

    @Value("${app.appid}")
    private String appid;
    @Value("${film.appKey}")
    private  String appKey ;
    @Value("${film.appSecret}")
    private  String appSecret ;
    @Value("${wxpay.notifyUrl}")
    private  String notifyUrl ;




    /**
     * 查询题库列表
     */

    @GetMapping("/order/list")
    @ApiOperation("查询订单列表")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "pageNum",value = "当前页码" ,dataType = "int", paramType = "query", required = false),
            @ApiImplicitParam(name = "pageSize",value = "每页数据量" , dataType = "int", paramType = "query", required = false),
    })
    public ResponsePageInfo<FilmOrder> list(@ModelAttribute FilmOrderListDTO filmOrderListDTO){

        String openid = SecurityUtils.getLoginUser().getStudent().getOpenid();
        startPage();
        filmOrderListDTO.setOpenid(openid);
        List<FilmOrder> list = filmOrderService.selectOrderList(filmOrderListDTO);
        return toResponsePageInfo(list);
    }

    /**
     * 微信支付获取预订单id
     */
    @PostMapping(value = "/prepareOrder")
    @ApiOperation("微信支付获取预订单id")
    public Response<JSONObject> getPrepareOrderId(@RequestBody FilmOrderCreateDTO filmOrderCreateDTO) throws Exception{
        HttpPost httpPost = new HttpPost(wxpayConfig.getV3url());
        httpPost.addHeader("Accept", "application/json");
        httpPost.addHeader("Content-type","application/json; charset=utf-8");

        ByteArrayOutputStream bos = new ByteArrayOutputStream();
        ObjectMapper objectMapper = new ObjectMapper();
        UserInfo student = SecurityUtils.getLoginUser().getStudent();

        //1 本地创建订单
        FilmOrder order = new FilmOrder();
        Long id = student.getId();
        String orderCode = getOrderCode(id);
        order.setOutTradeNo(orderCode);
        order.setOpenid(student.getOpenid());
        dictDataService.selectDictLabel("goods_type",filmOrderCreateDTO.getGoodsType());
        order.setGoodsType(filmOrderCreateDTO.getGoodsType());
        order.setGoodsPictureUrl(filmOrderCreateDTO.getGoodsPictureUrl());
        FilmOrderJsonData filmOrderJsonData = new FilmOrderJsonData();
        filmOrderCreateDTO.setThirdOrderId(orderCode);
        filmOrderJsonData.setFilmOrderCreateDTO(filmOrderCreateDTO);
        //通过影院api 获取单票价格
        Integer price = getPrice(filmOrderCreateDTO.getCinemaId(),filmOrderCreateDTO.getShowId(),filmOrderJsonData);
        int number = filmOrderCreateDTO.getSeat().split(",").length;
        order.setOrderDataJson(JSONObject.toJSONString(filmOrderJsonData));
        filmOrderService.save(order);



        // 2 调用微信接口生成预订单
        ObjectNode rootNode = objectMapper.createObjectNode();
        rootNode.put("mchid",wxpayConfig.getMerchantId())
                .put("appid", appid)
                .put("description", filmOrderCreateDTO.getGoodsType())
                .put("notify_url", notifyUrl)
                .put("out_trade_no", orderCode);
        rootNode.putObject("amount")
                .put("total", price*number);
        rootNode.putObject("payer")
                .put("openid", student.getOpenid());
        objectMapper.writeValue(bos, rootNode);
        httpPost.setEntity(new StringEntity(bos.toString("UTF-8")));
        HttpResponse response = httpClient.execute(httpPost);
        String bodyAsString = EntityUtils.toString(response.getEntity());
        System.out.println(bodyAsString);
        if (JSONObject.parseObject(bodyAsString).get("prepay_id") == null){
            throw new CustomException(JSONObject.parseObject(bodyAsString).get("message").toString());
        }


        //3 生成前端小程序调用调起微信支付所需参数
        String  packageStr = "prepay_id="+JSONObject.parseObject(bodyAsString).get("prepay_id");
        JSONObject jsonObject = new JSONObject();
        jsonObject.put("package",packageStr);
        String nonce_str = RandomStringUtils.randomAlphanumeric(32);
        jsonObject.put("nonceStr",nonce_str);
        long timestamp =  System.currentTimeMillis()/1000;
        jsonObject.put("timeStamp",String.valueOf(timestamp));
        jsonObject.put("signType","RSA");
        StringBuffer sb = new StringBuffer();
        sb.append(appid + "\n");
        sb.append(timestamp + "\n");
        sb.append(nonce_str + "\n");
        sb.append(packageStr+ "\n");
        System.out.println(sb);

        File file = new ClassPathResource("wechatpay/apiclient_key.pem").getFile();
        String realPath =file.getAbsolutePath();
        PrivateKey privateKey = CommonUtils.getPrivateKey(realPath);
        // 进行签名服务
        Signature signature = Signature.getInstance("SHA256withRSA");
        signature.initSign(privateKey);
        signature.update(sb.toString().getBytes("UTF-8"));
        byte[] signedData = signature.sign();
        String base64Str =  Base64.getEncoder().encodeToString(signedData);
        jsonObject.put("paySign",base64Str);
        jsonObject.put("outTradeNo",orderCode);
        return Response.success(jsonObject);

    }



    private Integer getPrice(String cinemaId, String showId, FilmOrderJsonData filmOrderJsonData) {

        FilmDTO filmDTO = new FilmDTO();
        filmDTO.setUrl("movieapi/movie-info/get-schedule-list");
        StringBuffer paramData =  new StringBuffer();
        paramData.append("cinemaId="+cinemaId);
        filmDTO.setParamData(paramData.toString());
        String result = filmOrderService.excuteFilmApi(filmDTO);
        JSONObject jsonObject = JSONObject.parseObject(result);
        JSONObject  data = jsonObject.getJSONObject("data");
        JSONObject  discountRule = data.getJSONObject("discountRule");
        JSONArray list = data.getJSONArray("list");
        List<CinemaData> cinemaData = list.toJavaList(CinemaData.class);
        List<CinemaData> listdata = cinemaData.stream().filter(a -> showId.equals(a.getShowId())).collect(Collectors.toList());
        CinemaData realCinemaData = listdata.get(0);
        filmOrderJsonData.setCinemaData(realCinemaData);
        Integer netPrice = realCinemaData.getNetPrice();
        Double upDiscountRate = discountRule.getDouble("upDiscountRate");
        Double downDiscountRate = discountRule.getDouble("downDiscountRate");
        String film_discount = configService.selectConfigByKey("film_discount");
        Double  chosenDiscount = netPrice >3900 ? upDiscountRate:downDiscountRate;
        Double aDouble = Double.valueOf(film_discount);
        Double v = netPrice * chosenDiscount * aDouble;

        return v.intValue();


    }






    /**
     * 以form表单形式提交数据，发送post请求
     * @explain
     *   1.请求头：httppost.setHeader("Content-Type","application/x-www-form-urlencoded")
     *   2.提交的数据格式：key1=value1&key2=value2...
     * @return 服务器返回数据
     */
    @PostMapping(value = "/fileCommonApi")
    @ApiOperation("电影通用接口")
    public  Response<JSONObject>  httpPostWithForm(@RequestBody FilmDTO filmDTO) throws IOException {

        //检查api
        if (Arrays.asList(FILM_BLACK_LIST).contains(filmDTO.getUrl())){
            throw new CustomException("该api不允许用户直接访问");
        }
        // 用于接收返回的结果
        String resultData = filmOrderService.excuteFilmApi(filmDTO);
        return Response.success(JSONObject.parseObject(resultData));
    }



    /**
     * 获取电影订单详细信息
     */
    @GetMapping(value = "/{outTradeNo}")
    @ApiOperation("获取电影订单详细信息")
    public Response<FilmOrder> getInfo(
            @ApiParam(name = "outTradeNo", value = "电影订单参数", required = true)
            @PathVariable("outTradeNo") String outTradeNo
    ){
        FilmOrder filmOrder = filmOrderService.getByOutTradeNo(outTradeNo);
        if (!filmOrder.getOpenid().equals(SecurityUtils.getLoginUser().getStudent().getOpenid())){
            throw new CustomException("该用户无访问此订单权限");
        }
        return Response.success();
    }







//    /**
//     * 微信支付查单接口
//     */
//    @GetMapping(value = "/wxorder/{outTradeNo}")
//    @ApiOperation("根据订单号查询订单状态")
//    public Response<String> getOrderInfo(
//            @ApiParam(name = "outTradeNo", value = "商户订单号", required = true)
//            @PathVariable("outTradeNo") String outTradeNo
//    ) throws IOException {
//        HttpGet get = new HttpGet("https://api.mch.weixin.qq.com/v3/pay/transactions/out-trade-no/"
//                +outTradeNo+"?mchid="+wxpayConfig.getMerchantId());
//        get.addHeader("Accept", "application/json");
//        HttpResponse response = httpClient.execute(get);
//        String bodyAsString = EntityUtils.toString(response.getEntity());
//        JSONObject jsonObject = JSONObject.parseObject(bodyAsString);
//        int statusCode = response.getStatusLine().getStatusCode();
//        if (statusCode == 200) {
//            return Response.success(jsonObject.getString("trade_state"));
//        } else {
//            throw new CustomException(EntityUtils.toString(response.getEntity()));
//        }
//    }















}
